/***********************************************************************
 * File: TM_sum.c
 * (Extracted from
 * "$Id: TM_sum.c,v 1.1 2007/10/30 09:35:19 Administrator Exp Administrator $")
 *
 * Abstracts:
 *  This C source code implements a test program for buffer_checker.c.
 *  This works as a Turing's machine that adds positive numbers.
 *  If You like to try this, type the following.  This prints "15".
 *	$ TM_sum 1 2 3 4 5
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

static int debug = 0; /* switch for debugging: 0 means no-debug. */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

/* At first, API users must include the following header file. */
#include <buffer_checker.h>


/* Second, API users must define the buffer structure layout. */
	/* ... But, this program doesn't need the buffer.
	 * Instead of it, this program needs a status structure.
	 * And then, this program needs a function that returns the
	 * address of a cell of tape under the head.
	 */
typedef enum TM_sum_status {
	  INIT	= 0		/* init */
	, SEEK_END_OF_ADDEE	/* seoa */
	, SEEK_BEGIN_OF_TAPE	/* sbot */
} TM_sum_status_e;	/* for `status' of `Tape_head_status_t' */

typedef struct tape_head_status_t {
#	define CELL_SZ	2	/* size of a cell of tape is 2 bytes */
#	define CELL_0	"0"	/* value of zero-ed cell */
#	define CELL_1	"1"	/* value of one-ed cell */
	size_t		head;	/* the position of tape-head */
	TM_sum_status_e	status;	/*  the internal status of TM-sum */
} Tape_head_status_t;	/* status and tape-head structure */

static void *get_cell_addr(void	 *buffer
			, ssize_t fld_offset
			, size_t *fldszp
			, void	 *user_cntxt
			); /* returns address of a cell at tape-head */


/* Third, API users must define the prototypes matched or unmatched
 * callbacks
 */
static int init_is_0_Noop_Term_init(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*passed
				);

static int init_is_1_Wrt0_Next_seoa(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*passed
				);

static int seoa_is_0_Wrt1_Prev_sbot(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*passed
				);

static int seoa_is_1_Noop_Next_seoa(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*passed
				);

static int sbot_is_0_Next_Term_init(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*passed
				);

static int sbot_is_1_Noop_Prev_sbot(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*passed
				);


/* Forth, API users must describe the buffer-checker configuration.
 *
 * 1) Buffer_checker_fld() macro is replaced offsetof() and fld_sizeof()
 *   macros.
 * 2) The second member (NULL) is a pointer to function that returns
 *   the field address in the buffer.
 * 3) Buffer_checker_RE() macro is replaced `RE, NULL' C-describe.
 *   And, Buffer_checker_ANY_MATCH_RE macro is replaced `"^.*$", NULL'.
 * 4) The forth and fifth members are callbacks at matched or unmatched.
 */
static Buffer_checker_t defines[] = {
  { /* INIT */
  	  0, CELL_SZ, get_cell_addr, Buffer_checker_RE(CELL_0
	, init_is_0_Noop_Term_init, init_is_1_Wrt0_Next_seoa)
	}

, { /* SEEK_END_OF_ADDEE */
  	  0, CELL_SZ, get_cell_addr, Buffer_checker_RE(CELL_0
	, seoa_is_0_Wrt1_Prev_sbot, seoa_is_1_Noop_Next_seoa)
	}

, { /* SEEK_BEGIN_OF_TAPE */
  	  0, CELL_SZ, get_cell_addr, Buffer_checker_RE(CELL_0
	, sbot_is_0_Next_Term_init, sbot_is_1_Noop_Prev_sbot)
	}
};
#define DEFINES_CNT (sizeof(defines) / sizeof(defines[0]))


/* Fifth, API users must implement the callbacks and utilities. */
static void dump_tape(char *tape, size_t tape_sz)
{
	size_t	i;

	if (debug) {
		for (i = 0; i < tape_sz; i += CELL_SZ) {
			printf("%d-th cell: %s\n", i, tape + i);
			fflush(stdout);
		}
		printf("tape_sz = %lu.\n"
			, (unsigned long)tape_sz
			); fflush(stdout);
	}

	return;
}

static void dump_internal(Tape_head_status_t *internal)
{
	if (debug) {
		printf("internal->head = %d, internal->status = %d "
			"(INIT = %d, SEEK_END_OF_ADDEE = %d, "
			"SEEK_BEGIN_OF_TAPE = %d).\n"
			,	internal->head,	internal->status
			, INIT,			SEEK_END_OF_ADDEE
			,		SEEK_BEGIN_OF_TAPE
			);
		fflush(stdout);
	}

	return;
}

#define ILLEGAL_INTERNAL_ARG					\
	( (internal == NULL)					\
		|| internal->status > SEEK_BEGIN_OF_TAPE	\
	)

static void *get_cell_addr(void *buffer
			, ssize_t fld_offset
			, size_t *fldszp
			, void *user_cntxt
			)
{
	Tape_head_status_t *internal = (Tape_head_status_t *)user_cntxt;

	/* test arguments and internal status */
	if ((((buffer == NULL)
	|| fld_offset != 0)
	|| fldszp == NULL)
	|| ILLEGAL_INTERNAL_ARG) {
		return	NULL;	/* !!FATAL!! Illegal arguments
				 * or Illegal internal status
				 */
	}

	/* returns address of the cell under the tape-head */
	return	(void *)((char *)buffer + internal->head);
}

#define	NOOP
#define WRT0(CELLP)	(strncpy((char *)(CELLP),CELL_0,CELL_SZ))
#define WRT1(CELLP)	(strncpy((char *)(CELLP),CELL_1,CELL_SZ))
#define NEXT(INTERNAL)	((INTERNAL)->head += CELL_SZ)
#define PREV(INTERNAL)	((INTERNAL)->head -= CELL_SZ)
#define	TERM	return	BCR_Terminate

#define ILLEGAL_CALLBACKS_ARGS(STAT)			\
{		if (((((checker_index != (STAT))	\
		|| buffer == NULL)			\
		|| buffer_sz <= CELL_SZ * 2)		\
		|| fldp == NULL)			\
		|| ILLEGAL_INTERNAL_ARG) {		\
			return BCR_Unmatched;		\
		}					\
}

static int init_is_0_Noop_Term_init(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*user_cntxt
				)
{
	Tape_head_status_t *internal = (Tape_head_status_t *)user_cntxt;

	/* test arguments and internal status */
	ILLEGAL_CALLBACKS_ARGS(INIT);

	if (debug) {
		printf("***DBG: init_is_1_Wrt0_Next_seoa():\n");
		fflush(stdout);
	} dump_internal(internal);

	NOOP;
	internal->status = (TM_sum_status_e)checker_index;
						/* stay at "init" */
	TERM; /* Terminate this Turing-machine. */
}

static int init_is_1_Wrt0_Next_seoa(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*user_cntxt
				)
{
	Tape_head_status_t *internal = (Tape_head_status_t *)user_cntxt;

	/* test arguments and internal status */
	ILLEGAL_CALLBACKS_ARGS(INIT);

	if (debug) {
		printf("***DBG: init_is_1_Wrt0_Next_seoa():\n");
		fflush(stdout);
	} dump_internal(internal);

	WRT0(fldp);
	NEXT(internal);
	return	internal->status = SEEK_END_OF_ADDEE;
					/* success to "seoa" */
}

static int seoa_is_0_Wrt1_Prev_sbot(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*user_cntxt
				)
{
	Tape_head_status_t *internal = (Tape_head_status_t *)user_cntxt;

	/* test arguments and internal status */
	ILLEGAL_CALLBACKS_ARGS(SEEK_END_OF_ADDEE);

	if (debug) {
		printf("***DBG: seoa_is_0_Wrt1_Prev_sbot():\n");
		fflush(stdout);
	} dump_internal(internal);

	WRT1(fldp);
	PREV(internal);
	return	internal->status = SEEK_BEGIN_OF_TAPE;
					/* success to "sbot" */
}

static int seoa_is_1_Noop_Next_seoa(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*user_cntxt
				)
{
	Tape_head_status_t *internal = (Tape_head_status_t *)user_cntxt;

	/* test arguments and internal status */
	ILLEGAL_CALLBACKS_ARGS(SEEK_END_OF_ADDEE);

	if (debug) {
		printf("***DBG: seoa_is_1_Noop_Next_seoa():\n");
		fflush(stdout);
	} dump_internal(internal);

	NOOP;
	NEXT(internal);
	return	internal->status = (TM_sum_status_e)checker_index;
						/* stay at "seoa" */
}

static int sbot_is_0_Next_Term_init(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*user_cntxt
				)
{
	Tape_head_status_t *internal = (Tape_head_status_t *)user_cntxt;

	/* test arguments and internal status */
	ILLEGAL_CALLBACKS_ARGS(SEEK_BEGIN_OF_TAPE);

	if (debug) {
		printf("***DBG: sbot_is_0_Next_Term_init():\n");
		fflush(stdout);
	} dump_internal(internal);

	NEXT(internal);
	internal->status = INIT; /* success to "init" */
	TERM; /* Terminate this Turing-machine. */
}

static int sbot_is_1_Noop_Prev_sbot(size_t	 checker_index
				, void		*buffer
				, size_t	 buffer_sz
				, void		*fldp
				, void		*user_cntxt
				)
{
	Tape_head_status_t *internal = (Tape_head_status_t *)user_cntxt;

	/* test arguments and internal status */
	ILLEGAL_CALLBACKS_ARGS(SEEK_BEGIN_OF_TAPE);

	if (debug) {
		printf("***DBG: sbot_is_1_Noop_Prev_sbot():\n");
		fflush(stdout);
	} dump_internal(internal);

	NOOP;
	PREV(internal);
	return	internal->status = (TM_sum_status_e)checker_index;
						/* stay at "sbot" */
}


/* At last, API users can invoke the API-s;
 *			init_buffer_checker(), exec_buffer_checker(),
 *			and finish_buffer_checker()
 */
static void usage()
{
	fprintf(stderr, "Usage: TM_sum num num [num...]\n"
			"    num must be the positive integer.\n"
		); fflush(stderr);
	return;
}

int main(int ac, char **av)
{
	char	*tape = NULL, *tp = NULL;
	size_t	tape_sz = 0L;
	int	i, exit_code = 1;
	size_t	executed_cnt = 0;
	Buffer_checker_result_t	result;
	Tape_head_status_t	internal;

	/* test command options */
	if (ac < 3) {
		usage();
		exit(exit_code); return exit_code; /* Usage Error! */
	} ++exit_code;

	/* initialize the tape according to command options... */
	for (i = 1; i < ac; ++i) {
		long	n;
		errno	= 0; /* initialize errno */

		/* get the number */
		n = strtol((const char *)av[i], NULL, 10);

		/* test result of `strtol(3)' */
		if (errno == ERANGE
		|| n == 0 || n == LONG_MAX || n == LONG_MIN) {
			fprintf(stderr, "\"%s\" is illegal number.\n"
				,	av[i]
				); fflush(stderr);
			exit(exit_code); return exit_code;
						/* Illegal Number! */
		}

		/* allocate memory for the tape */
		if ((tape = (char *)realloc(tape
					, tape_sz + CELL_SZ * (n + 1)
					)) == NULL) {
			fprintf(stderr
			, "Can't allocate memory for the %d-th arg "
							"\"%s\".\n"
			,				i
			,				av[i]
			); fflush(stderr);
			exit(exit_code); return exit_code;
					/* Failed to realloc(3) */
		}

		/* set a positive number to the tape */
		tp	= tape + tape_sz;
		tape_sz	+= CELL_SZ * (n + 1);
		while (n-- > 0) {
			WRT1(tp); tp += CELL_SZ;
		}
		WRT0(tp); tp += CELL_SZ;
	} ++exit_code;

	if (debug) {
		dump_buffer_checker_regex_preg_list(debug);
		fprintf(stderr, "\n"); fflush(stderr);
	}

	/* initialize the `defines' */
	if ((result = init_buffer_checker(defines
					, DEFINES_CNT
					, tape_sz
					)) != BCR_Continue) {
		fprintf(stderr
		, "!!FATAL!! init_buffer_checker(%p, %u, %lu) returns "
			" %d (expected BCR_Continue = %d).\n"
		, (void *)defines, DEFINES_CNT, (unsigned long)tape_sz
		,	result,			BCR_Continue
		); fflush(stderr);
		goto	FATAL;
	} ++exit_code;

	if (debug) {
		dump_buffer_checker_regex_preg_list(debug);
		fprintf(stderr, "\n"); fflush(stderr);
	}

	/* initialize the tape-head and internal status */
	internal.head	= 0;
	internal.status	= INIT;

	if (debug) {
		printf(">>> initialized tape contains...\n");
		fflush(stdout);
	} dump_tape(tape, tape_sz); dump_internal(&internal);

	/* run the Turing-machine... */
	for (result = BCR_Continue, i = 2
	; i < ac && (result == BCR_Continue
			|| result == BCR_Terminate
			|| result >= 0
			)
	; ++i) {
		result = exec_buffer_checker(tape
					, tape_sz
					, defines
					, DEFINES_CNT
					, &internal
					, &executed_cnt
					);
	}

	/* count up the cells those are set "1". */
	for (i = 0
	; strncmp((tape + (internal.head + CELL_SZ * i))
		, CELL_1
		, CELL_SZ) == 0
	; ++i) {
		/* the last `i' means the counts of those. */ ;
	}

	/* print the sum of positive integers */
	printf("%d\n", i); fflush(stdout);

	if (debug) {
		fprintf(stderr
		, "=== exec_buffer_checker(%p, %lu, %p, %u, %p, %p) "
		  "returns %d (BCR_Continue = %d, BCR_Terminate = %d, "
		  "BCR_Unmatched = %d). executed_cnt = %d.\n"
		, tape, (unsigned long)tape_sz, (void *)defines
		, DEFINES_CNT,	(void *)&internal, (void *)&executed_cnt
		,	result,		BCR_Continue, BCR_Terminate
		,			BCR_Unmatched,	executed_cnt
		);
		fflush(stderr);

		printf("<<< calculated tape contains...\n");
		fflush(stdout);
	} dump_tape(tape, tape_sz); dump_internal(&internal);

	/* release the `defines' allocated memories */
	if ((result = finish_buffer_checker(defines
					, DEFINES_CNT
					)) != BCR_Continue) {
		fprintf(stderr
		, "!!FATAL!! finish_buffer_checker(%p, %u) returns %d "
			"(expected BCR_Continue = %d).\n"
		,		(void *)defines, DEFINES_CNT, result
		,				BCR_Continue
		); fflush(stderr);
	}
	else {
		if (debug) {
			dump_buffer_checker_regex_preg_list(debug);
			fprintf(stderr, "\n"); fflush(stderr);
		}

		exit_code	= 0; /* Normal Exit-code */
	}

FATAL:
	/* release the tape allocated memories */
	free(tape); tape = NULL;

	exit(exit_code); return exit_code; /* Exit Program */
}
