/***********************************************************************
 * File: unit_test.c (for simple_test.c)
 * 	(Extracted from
 *	   "$Id: unit_test.c,v 1.1 2007/10/27 20:54:35 Administrator Exp Administrator $")
 *
 * Abstracts:
 *	  This C source implements the unit test program
 *	for the simple hashing API-s are implemented 'simple_hashing.c'.
 *
 * Copyright (C) 2005-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "simple_hashing.h" /* simple hashing definitions */

#include <time.h>	/* for 'gettimeofday()' */
#include <sys/time.h>	/* for 'gettimeofday()' */


/*
 * This private method compares the keys and the datum.
 * If the datum is matched to keys, this returns zero.
 * Otherwise, this returns non-zero (same as 'strcmp()').
 *
 * Arguments means...
 *	keys	- pointer to the key string array
 *	n_keys	- size of 'keys' string array
 *	elem	- pointer to the sorted datum array
 */
static int cmp_keys(char **keys, size_t n_keys, void *elem)
{
	int	result = INT_MIN; /* default return value is negative */
	char	**element = (char **)elem;
	size_t	i = 0;

	errno	= EINVAL; /* default 'errno' means illegal arguments */

	/* compare every key strings... */
	for (i = 0; i < n_keys; ++i) {
		errno	= 0; /* set 'errno' to normal. */
		if ((result = strcmp(keys[i], element[i])) != 0) {
			break; /* this key isn't matched to the datum */
		}
	}

	return	result; /* result of comparison */
}


/*
 * This private method returns the new datum that has specified keys.
 * If this failed to allocate the datum memory, this terminate unit test
 * process (exit code is 255).
 *
 * Arguments means...
 *	keys	- pointer to the key string array
 *	n_keys	- size of 'keys' string array
 */
static char **make_elem(char **keys, size_t n_keys)
{
	size_t	i;
	char	**elemp
		, **elem = (char **)malloc(sizeof(keys) * n_keys)
		;

	/* test failed to allocate the datum area? */
	if (elem == NULL) {
FATAL:		fprintf(stderr, "!!FATAL!! 'make_elem(%p, %lu) "
			"returns %p. 'errno' = %d.\n"
			,	(void *)keys, (unsigned long)n_keys
			,	(void *)elem,		errno
			); (void)fflush(stderr);
		exit(255); /* Unit Test fatal errno! */
	}

	/* copy the keys to the new allocated datum area. */
	for (i = 0, elemp = elem; i < n_keys; ++i) {
		if ((elemp[i] = (char *)malloc(strlen(keys[i]) + 1))
							== NULL) {
			/* failed to allocate key copying area */
			goto FATAL;
		}
		strcpy(elemp[i], keys[i]); /* copy the key string */
	}

	return	elem; /* the new datum that copied key strings */
}


/*
 * This private method release the memories that is allocated
 * by 'make_elem()'.
 * This returns nothing (void).
 *
 * Arguments means...
 *	elem	- address of the pointer to the sorted datum array
 *	n_keys	- size of key strings in the datum
 */
static void destroy_elem(char ***elem, size_t n_keys)
{
	if (*elem != NULL) {
		char	**elem_self = *elem;
		while (n_keys-- > 0) {
			free((*elem)[n_keys]); 
		}
		free(elem_self); *elem = NULL;
	}

	return;
}


/*
 * The unit test program entry.
 *
 * The test set is...
 *	command	:
 *	option	: means
 *	     -I	: test illegal arguments detections
 *	     -N	: test insert, replace and delete the numeric keys
 *	     -A	: test insert, replace and delete the alphabetic keys
 *	     -S	: test insert, replace and delete the symbolic keys
 *	     -H	: test insert, replace and delete the hankaku-kana keys
 *	     -P : execute performance test (insert, replace and search
 *		  172980 datum)
 *	 (none)	: test all
 */
int main(int ac, char **av)
{
#	define	KEY_LEN 5
#	define	GET_KEYS_N(LIMIT,KEY_CHARS_SZ)			\
			(2 * (LIMIT) * (KEY_CHARS_SZ) * (KEY_CHARS_SZ))
	int	set_hint_max_insert_cnt;
	Hashtable_t *hashtable, *r_hashtable, *all_inserted_hashtable;
	int	exit_code = 0, err_num, state, pos_cmp_result = 0;
	Hashtable_cursor_t	*cursor = NULL;
	struct timeval	start_tv, end_tv, elapsed_tv;
	Hashtable_extend_hint_t	*hint = NULL, hints[2] = {
		  { (ssize_t)(LONG_MIN + 1L)
		  	, HASHTABLE_RECOMMENDED_ELEMS_BASE_CNT
			, (ssize_t)HASHTABLE_RECOMMENDED_ELEMS_EXT_CNT
			}
		, { HASHTABLE_SYS_MAX_INSERT_CNT, 1, 1 }
		};
	char	key0[KEY_LEN], *keys[1] = { "0000" }, **elem
		, **repl_elem, **all_elem
		, *keys1[1] = { "2222" }
		, *keys2[1] = { "4444" }
		, *keys3[1] = { "6666" }
		, *array_elem1[1] = { "1111" }
		, *array_elem2[1] = { "3333" }
		, *array_elem3[1] = { "5555" }
		, *keypool, *keyp;
	void	*array[3], *NULLp = NULL;
	size_t	n_keys = (sizeof(keys) / sizeof(keys[0]))
		, icnt, allocsz, key_chars_sz, i[4], j, limit;
	ssize_t	ucnt, dcnt, depth = 1009
		, n_array = (ssize_t)(sizeof(array) / sizeof(array[0]))
		, pos = 0;

	/* initialize array[] */
	array[0] = (void *)array_elem1;
	array[1] = (void *)array_elem2;
	array[2] = (void *)array_elem3;

	/* Test illegal arguments detection */
	if (ac == 1 || strcmp(av[1], "-I") == 0) {
		printf("+++UT:L%d: Test illegal arguments...\n"
			, __LINE__
			); (void)fflush(stdout);
		errno	= 0; exit_code	= 1;

		state = bsearch_array(NULL, NULL, (ssize_t)-1L
				, NULL, 0UL, NULL, NULL);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p, %lu, %p, %p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__,	NULLp, NULLp, -1L
			, NULLp, 0UL
			, NULLp, NULLp
			,	state,		err_num
			); (void)fflush(stdout);
		if (state != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, NULL, (ssize_t)-1L
				, NULL, 0UL, NULL, NULL);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p, %lu, %p, %p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__, (void *)cmp_keys, NULLp, -1L
			, NULLp, 0UL
			, NULLp, NULLp
			,	state,		err_num
			); (void)fflush(stdout);
		if (state != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, (ssize_t)-1L
				, NULL, 0UL, NULL, NULL);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p, %lu, %p, %p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array, -1L
			, NULLp, 0UL
			, NULLp, NULLp
			,	state,		err_num
			); (void)fflush(stdout);
		if (state != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, NULL, 0UL, NULL, NULL);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p, %lu, %p, %p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, NULLp, 0UL
			, NULLp, NULLp
			,	state,		err_num
			); (void)fflush(stdout);
		if (state != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, keys, 0UL, NULL, NULL);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p, %lu, %p, %p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)keys, 0UL
			, NULLp, NULLp
			,	state,		err_num
			); (void)fflush(stdout);
		if (state != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, keys, n_keys, NULL, NULL);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p, %lu, %p, %p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)keys, (unsigned long)n_keys
			, NULLp, NULLp
			,	state,		err_num
			); (void)fflush(stdout);
		if (state != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, keys, n_keys, &pos, NULL);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p, %lu, %p, %p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)keys, (unsigned long)n_keys
			, (void *)&pos, NULLp
			,	state,		err_num
			); (void)fflush(stdout);
		if (state != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, keys, n_keys, &pos, &pos_cmp_result);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p (%s), %lu, %p, %p) "
			"returns %d. 'errno' = %d. "
			"pos = %ld. pos_cmp_result = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)keys, keys[0], (unsigned long)n_keys
			, (void *)&pos, (void *)&pos_cmp_result
			,	state,		err_num
			,	(long)pos,	pos_cmp_result
			); (void)fflush(stdout);
		if (state != 0 || err_num != 0
		|| pos != 0 || pos_cmp_result != INT_MIN) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
			, array_elem1, n_keys, &pos, &pos_cmp_result);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p (%s), %lu, %p, %p) "
			"returns %d. 'errno' = %d. "
			"pos = %ld. pos_cmp_result = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)array_elem1, array_elem1[0]
			, (unsigned long)n_keys
			, (void *)&pos, (void *)&pos_cmp_result
			,	state,		err_num
			,	(long)pos,	pos_cmp_result
			); (void)fflush(stdout);
		if (state != 1 || err_num != 0
		|| pos != 0 || pos_cmp_result != 0) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, keys1, n_keys, &pos, &pos_cmp_result);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p (%s), %lu, %p, %p) "
			"returns %d. 'errno' = %d. "
			"pos = %ld. pos_cmp_result = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)keys1, keys1[0], (unsigned long)n_keys
			, (void *)&pos, (void *)&pos_cmp_result
			,	state,		err_num
			,	(long)pos,	pos_cmp_result
			); (void)fflush(stdout);
		if (state != 0 || err_num != 0
		|| pos != 0 || pos_cmp_result != 1) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
			, array_elem2, n_keys, &pos, &pos_cmp_result);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p (%s), %lu, %p, %p) "
			"returns %d. 'errno' = %d. "
			"pos = %ld. pos_cmp_result = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)array_elem2, array_elem2[0]
			, (unsigned long)n_keys
			, (void *)&pos, (void *)&pos_cmp_result
			,	state,		err_num
			,	(long)pos,	pos_cmp_result
			); (void)fflush(stdout);
		if (state != 1 || err_num != 0
		|| pos != 1 || pos_cmp_result != 0) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, keys2, n_keys, &pos, &pos_cmp_result);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p (%s), %lu, %p, %p) "
			"returns %d. 'errno' = %d. "
			"pos = %ld. pos_cmp_result = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)keys2, keys2[0], (unsigned long)n_keys
			, (void *)&pos, (void *)&pos_cmp_result
			,	state,		err_num
			,	(long)pos,	pos_cmp_result
			); (void)fflush(stdout);
		if (state != 0 || err_num != 0
		|| pos != 1 || pos_cmp_result != 1) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
			, array_elem3, n_keys, &pos, &pos_cmp_result);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p (%s), %lu, %p, %p) "
			"returns %d. 'errno' = %d. "
			"pos = %ld. pos_cmp_result = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)array_elem3, array_elem3[0]
			, (unsigned long)n_keys
			, (void *)&pos, (void *)&pos_cmp_result
			,	state,		err_num
			,	(long)pos,	pos_cmp_result
			); (void)fflush(stdout);
		if (state != 1 || err_num != 0
		|| pos != 2 || pos_cmp_result != 0) {
			exit(exit_code);
		}

		state = bsearch_array(cmp_keys, array, n_array
				, keys3, n_keys, &pos, &pos_cmp_result);
		err_num	= errno;
		printf("UT:L%d: bsearch_array(%p, %p, %ld, "
					"%p (%s), %lu, %p, %p) "
			"returns %d. 'errno' = %d. "
			"pos = %ld. pos_cmp_result = %d.\n"
			, __LINE__, (void *)cmp_keys, (void *)array
			, (long)n_array
			, (void *)keys3, keys3[0]
			, (unsigned long)n_keys
			, (void *)&pos, (void *)&pos_cmp_result
			,	state,		err_num
			,	(long)pos,	pos_cmp_result
			); (void)fflush(stdout);
		if (state != 0 || err_num != 0
		|| pos != 2 || pos_cmp_result != 1) {
			exit(exit_code);
		}

		hashtable = create_hash_table(-1, NULL, NULL, hint);
		err_num	= errno;
		printf("UT:L%d: create_hash_table(%d, %p, %p, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	-1, NULLp, NULLp, (void *)hint
			, (void *)hashtable,	err_num
			); (void)fflush(stdout);
		if (hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		hashtable = create_hash_table(0, NULL, NULL, hint);
		err_num	= errno;
		printf("UT:L%d: create_hash_table(%d, %p, %p, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	0, NULLp, NULLp, (void *)hint
			, (void *)hashtable,	err_num
			); (void)fflush(stdout);
		if (hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		hashtable = create_hash_table(depth, NULL, NULL, hint);
		err_num	= errno;
		printf("UT:L%d: create_hash_table(%ld, %p, %p, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(long)depth, NULLp, NULLp
			, (void *)hint
			, (void *)hashtable,	err_num
			); (void)fflush(stdout);
		if (hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		hashtable = create_hash_table(depth, NULL
						, cmp_keys, hint);
		err_num	= errno;
		printf("UT:L%d: create_hash_table(%ld, %p, %p, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (long)depth, NULLp, (void *)cmp_keys
			, (void *)hint
			, (void *)hashtable,	err_num
			); (void)fflush(stdout);
		if (hashtable == NULL || err_num != 0) {
			exit(exit_code);
		}

		allocsz	= get_allocated_size_hash_table(NULL);
		err_num	= errno;
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu. 'errno' = %d.\n"
			, __LINE__,			NULLp
			,	(unsigned long)allocsz, err_num
			); (void)fflush(stdout);
		if (allocsz != 0 || err_num != EINVAL) {
			exit(exit_code);
		}

		allocsz	= get_allocated_size_hash_table(hashtable);
		err_num	= errno;
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu (sizeof(*hashtable) = %lu, "
			"hashtable->depth = %ld). "
			"'errno' = %d.\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)allocsz
			, (unsigned long)sizeof(*hashtable)
			,	(long)hashtable->depth
			,	err_num
			); (void)fflush(stdout);
		if (allocsz < sizeof(*hashtable) || err_num != 0) {
			exit(exit_code);
		}

		icnt = get_inserted_cnt_hash_table(NULL);
		err_num	= errno;
		printf("UT:L%d: get_inserted_cnt_hash_table(%p) "
			"returns %lu. 'errno' = %d.\n"
			, __LINE__,		NULLp
			, (unsigned long)icnt, err_num
			); (void)fflush(stdout);
		if (icnt != (size_t)ULONG_MAX || err_num != EINVAL) {
			exit(exit_code);
		}

		icnt = get_inserted_cnt_hash_table(hashtable);
		err_num	= errno;
		printf("UT:L%d: get_inserted_cnt_hash_table(%p) "
			"returns %lu. 'errno' = %d.\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)icnt,	err_num
			); (void)fflush(stdout);
		if (icnt != 0 || err_num != 0) {
			exit(exit_code);
		}

		ucnt = get_used_cnt_hash_table(NULL);
		err_num	= errno;
		printf("UT:L%d: get_used_cnt_hash_table(%p) "
			"returns %ld. 'errno' = %d.\n"
			, __LINE__,		NULLp
			,	(long)ucnt,	err_num
			); (void)fflush(stdout);
		if (ucnt != (ssize_t)LONG_MIN
		|| err_num != EINVAL) {
			exit(exit_code);
		}

		ucnt = get_used_cnt_hash_table(hashtable);
		err_num	= errno;
		printf("UT:L%d: get_used_cnt_hash_table(%p) "
			"returns %ld. 'errno' = %d.\n"
			, __LINE__,		(void *)hashtable
			, (long)ucnt,	err_num
			); (void)fflush(stdout);
		if (ucnt != 0 || err_num != 0) {
			exit(exit_code);
		}

		dcnt = get_duplicated_cnt_hash_table(NULL, NULL, 0UL);
		err_num	= errno;
		printf("UT:L%d: get_duplicated_cnt_hash_table(%p, %p, "
								"%lu) "
			"returns %lu. 'errno' = %d.\n"
			, __LINE__,		NULLp, NULLp, 0UL
			, (unsigned long)dcnt,		err_num
			); (void)fflush(stderr);
		if (dcnt != (ssize_t)LONG_MIN || err_num != EINVAL) {
			exit(exit_code);
		}

		dcnt = get_duplicated_cnt_hash_table(hashtable, NULL
								, 0UL);
		err_num	= errno;
		printf("UT:L%d:  get_duplicated_cnt_hash_table(%p, %p, "
								"%lu) "
			"returns %lu. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, NULLp, 0UL
			, (unsigned long)dcnt,		err_num
			); (void)fflush(stderr);
		if (dcnt != (ssize_t)LONG_MIN || err_num != EINVAL) {
			exit(exit_code);
		}

		dcnt = get_duplicated_cnt_hash_table(hashtable, keys
								, 0UL);
		err_num	= errno;
		printf("UT:L%d: get_duplicated_cnt_hash_table(%p, %p, "
								"%lu) "
			"returns %lu. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, 0UL
			, (unsigned long)dcnt,		err_num
			); (void)fflush(stderr);
		if (dcnt != (ssize_t)LONG_MIN || err_num != EINVAL) {
			exit(exit_code);
		}

		dcnt = get_duplicated_cnt_hash_table(hashtable, keys	
							, n_keys);
		err_num	= errno;
		printf("UT:L%d: get_duplicated_cnt_hash_table(%p, %p, "
								"%lu) "
			"returns %lu. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys
			, (unsigned long)dcnt,		err_num
			); (void)fflush(stderr);
		if (dcnt != 0 || err_num != 0) {
			exit(exit_code);
		}

		r_hashtable = delete_hash_table(NULL, NULL, 0UL, NULL);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,		NULLp, NULLp, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable = delete_hash_table(hashtable, NULL, 0UL
								, NULL);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, NULLp
			, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable = delete_hash_table(hashtable, keys, 0UL
								, NULL);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			);
		if (r_hashtable != NULL || err_num != ERANGE) {
			exit(exit_code);
		}

		r_hashtable = delete_hash_table(hashtable, keys, n_keys
								, NULL);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys, NULLp
			, (void *)r_hashtable,	err_num
			);
		if (r_hashtable != NULL || err_num != ENOENT) {
			exit(exit_code);
		}

		elem	= make_elem(keys, n_keys);
		r_hashtable = delete_hash_table(hashtable, keys, n_keys
								, elem);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys, (void *)elem
			, (void *)r_hashtable,	err_num
			);
		if (r_hashtable != NULL || err_num != ENOENT) {
			exit(exit_code);
		}

		r_hashtable= insert_hash_table(hashtable, keys, n_keys
							, elem);
		r_hashtable = delete_hash_table(hashtable, keys1, n_keys
								, NULL);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys, NULLp
			, (void *)r_hashtable,	err_num
			);
		if (r_hashtable != NULL || err_num != ENOENT) {
			exit(exit_code);
		}

		r_hashtable = delete_hash_table(hashtable, keys, n_keys
								, NULL);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys
			, NULLp
			, (void *)r_hashtable,	err_num
			);
		if (r_hashtable != hashtable || err_num != 0) {
			exit(exit_code);
		}
		destroy_elem(&elem, n_keys);

		r_hashtable= insert_hash_table(NULL, NULL, 0UL, NULL);
		err_num	= errno;
		printf("UT:L%d: insert_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,		NULLp, NULLp, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable= insert_hash_table(hashtable, NULL, 0UL
								, NULL);
		err_num	= errno;
		printf("UT:L%d: insert_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, NULLp, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable= insert_hash_table(hashtable, keys, 0UL
								, NULL);
		err_num	= errno;
		printf("UT:L%d: insert_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)keys
			, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable= insert_hash_table(hashtable, keys, n_keys
								, NULL);
		err_num	= errno;
		printf("UT:L%d: insert_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		elem	= make_elem(keys, n_keys);

		r_hashtable= insert_hash_table(hashtable, keys, n_keys
								, elem);
		err_num	= errno;
		printf("UT:L%d: insert_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys, (void *)elem
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable == NULL || err_num != 0) {
			exit(exit_code);
		}

		repl_elem	= make_elem(keys, n_keys);
		r_hashtable= insert_hash_table(hashtable, keys, n_keys
							, repl_elem);
		err_num	= errno;
		printf("UT:L%d: insert_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys
			, (void *)repl_elem
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != ESRCH) {
			exit(exit_code);
		}

		destroy_elem(&repl_elem, n_keys);

		repl_elem = (char **)get_elem_hash_table(NULL
							, NULL
							, 0UL
							);
		err_num	= errno;
		printf("UT:L%d: get_elem_hash_table(%p, %p, %lu) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,		NULLp, NULLp, 0UL
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		repl_elem = (char **)get_elem_hash_table(hashtable
							, NULL
							, 0UL
							);
		err_num	= errno;
		printf("UT:L%d: get_elem_hash_table(%p, %p, %lu) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, NULLp, 0UL
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		repl_elem = (char **)get_elem_hash_table(hashtable
							, keys
							, 0UL
							);
		err_num	= errno;
		printf("UT:L%d: get_elem_hash_table(%p, %p, %lu) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)keys, 0UL
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		repl_elem = (char **)get_elem_hash_table(hashtable
							, keys
							, n_keys
							);
		err_num	= errno;
		printf("UT:L%d: get_elem_hash_table(%p, %p, %lu) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)keys
			, (unsigned long)n_keys
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem == NULL || err_num != 0) {
			exit(exit_code);
		}

		r_hashtable = replace_hash_table(NULL, NULL, 0UL, NULL);
		err_num	= errno;
		printf("UT:L%d: replace_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,		NULLp, NULLp, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable = replace_hash_table(hashtable, NULL, 0UL
								, NULL);
		err_num	= errno;
		printf("UT:L%d: replace_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, NULLp, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable = replace_hash_table(hashtable, keys, 0UL
								, NULL);
		err_num	= errno;
		printf("UT:L%d: replace_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)keys
			, 0UL, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable = replace_hash_table(hashtable, keys, n_keys
								, NULL);
		err_num	= errno;
		printf("UT:L%d: replace_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)keys
			, (unsigned long)n_keys, NULLp
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		r_hashtable = replace_hash_table(hashtable, keys
						, n_keys, repl_elem);
		err_num	= errno;
		printf("UT:L%d: replace_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)keys
			, (unsigned long)n_keys, (void *)repl_elem
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != hashtable || err_num != 0) {
			exit(exit_code);
		}

		elem = repl_elem; repl_elem = make_elem(keys, n_keys);
		r_hashtable = replace_hash_table(hashtable, keys
						, n_keys, repl_elem);
		err_num	= errno;
		printf("UT:L%d: replace_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)keys
			, (unsigned long)n_keys, (void *)repl_elem
			, (void *)r_hashtable,	err_num
			); (void)fflush(stdout);
		if (r_hashtable != hashtable || err_num != 0) {
			exit(exit_code);
		}

		destroy_elem(&elem, n_keys); elem = repl_elem;

		repl_elem = (char **)get_elem_by_hashtable_cursor(
					  NULL
					, NULL
					, (Hashtable_cursor_dir_t)-1
					);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,			NULLp, NULLp, -1
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		repl_elem = (char **)get_elem_by_hashtable_cursor(
					  hashtable
					, NULL
					, (Hashtable_cursor_dir_t)-1
					);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, NULLp, -1
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		cursor	= NULL;
		repl_elem = (char **)get_elem_by_hashtable_cursor(
					  hashtable
					, &cursor
					, (Hashtable_cursor_dir_t)-1
					);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, -1
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem == NULL || err_num != 0) {
			exit(exit_code);
		}
		(void)get_elem_by_hashtable_cursor(hashtable, &cursor
					, HASHTABLE_CURSOR_DIR_DESTROY);

		cursor	= NULL;
		repl_elem = (char **)get_elem_by_hashtable_cursor(
			  hashtable
			, &cursor
			, (Hashtable_cursor_dir_t)(
				HASHTABLE_CURSOR_DIR_DESTROY + 1)
			);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_DESTROY + 1
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem == NULL || err_num != 0) {
			exit(exit_code);
		}
		(void)get_elem_by_hashtable_cursor(hashtable, &cursor
					, HASHTABLE_CURSOR_DIR_DESTROY);

		cursor	= NULL;
		repl_elem = (char **)get_elem_by_hashtable_cursor(
				  hashtable
				, &cursor
				, HASHTABLE_CURSOR_DIR_DESTROY
				);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_DESTROY
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem == NULL || err_num != 0) {
			exit(exit_code);
		}
		(void)get_elem_by_hashtable_cursor(hashtable, &cursor
					, HASHTABLE_CURSOR_DIR_DESTROY);

		cursor	= NULL;
		repl_elem = (char **)get_elem_by_hashtable_cursor(
				  hashtable
				, &cursor
				, HASHTABLE_CURSOR_DIR_PREVIOUS
				);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_PREVIOUS
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem == NULL || err_num != 0) {
			exit(exit_code);
		}
		(void)get_elem_by_hashtable_cursor(hashtable, &cursor
					, HASHTABLE_CURSOR_DIR_DESTROY);

		cursor	= NULL;
		repl_elem = (char **)get_elem_by_hashtable_cursor(
				  hashtable
				, &cursor
				, HASHTABLE_CURSOR_DIR_NEXT
				);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_NEXT
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem == NULL || err_num != 0) {
			exit(exit_code);
		}
		(void)get_elem_by_hashtable_cursor(hashtable, &cursor
					, HASHTABLE_CURSOR_DIR_DESTROY);

		cursor	= NULL;
		repl_elem = (char **)get_elem_by_hashtable_cursor(
				  hashtable
				, &cursor
				, HASHTABLE_CURSOR_DIR_CURRENT
				);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_CURRENT
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem == NULL || err_num != 0) {
			exit(exit_code);
		}

		repl_elem = (char **)get_elem_by_hashtable_cursor(
				  hashtable
				, &cursor
				, HASHTABLE_CURSOR_DIR_PREVIOUS
				);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_PREVIOUS
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		repl_elem = (char **)get_elem_by_hashtable_cursor(
				  hashtable
				, &cursor
				, HASHTABLE_CURSOR_DIR_NEXT
				);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_NEXT
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != NULL || err_num != EINVAL) {
			exit(exit_code);
		}

		repl_elem = (char **)get_elem_by_hashtable_cursor(
				  hashtable
				, &cursor
				, HASHTABLE_CURSOR_DIR_DESTROY
				);
		err_num	= errno;
		printf("UT:L%d: get_elem_by_hashtable_cursor(%p, %p, "
								"%d) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__, (void *)hashtable, (void *)&cursor
			, HASHTABLE_CURSOR_DIR_DESTROY
			, (void *)repl_elem,	err_num
			); (void)fflush(stdout);
		if (repl_elem != (char **)hashtable || err_num != 0) {
			exit(exit_code);
		}

		r_hashtable = delete_hash_table(hashtable, keys, n_keys
								, elem);
		err_num	= errno;
		printf("UT:L%d: delete_hash_table(%p, %p, %lu, %p) "
			"returns %p. 'errno' = %d.\n"
			, __LINE__,	(void *)hashtable, (void *)keys
			, (unsigned long)n_keys, (void *)elem
			,	(void *)r_hashtable,	err_num
			);
		if (r_hashtable == NULL || err_num != 0) {
			exit(exit_code);
		}

		destroy_elem(&elem, n_keys);

		state	= destroy_hash_table(NULL);
		err_num	= errno;
		printf("UT:L%d: destroy_hash_table(%p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__,		NULLp
			,	state,		err_num
		); (void)fflush(stdout);
		if (state != -1 || err_num != EINVAL) {
			exit(exit_code);
		}

		state	= destroy_hash_table(hashtable);
		hashtable = NULL;
		err_num	= errno;
		printf("UT:L%d: destroy_hash_table(%p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__,		(void *)hashtable
			,	state,		err_num
		); (void)fflush(stdout);
		if (state != 0 || err_num != 0) {
			exit(exit_code);
		}

		printf("+++UT:L%d: Test illegal arguments done!\n\n"
			, __LINE__
			); (void)fflush(stdout);
	} /* Test illegal arguments */

	/*
	 * create 'all_inserted_hashtable'
	 * for get_elem_by_hashtable_cursor()'
	 */
	all_inserted_hashtable = create_hash_table(depth
						, NULL
						, cmp_keys
						, hint
						);
	if (all_inserted_hashtable == NULL) {
		printf("!!FATAL!!:L%d:exit_code [%d]:"
			"create_hash_table(all_inserted_hashtable) "
			"returns %p(errno = %d).\n"
			,	__LINE__,	exit_code
			,	(void *)all_inserted_hashtable, errno
			); (void)fflush(stderr);
		exit(exit_code);
	}

	/* Test the 2000 numeric-string keys "0000" to "1999" */
	if (ac == 1 || strcmp(av[1], "-N") == 0) {
#		define	KEY_CHARS	"0123456789"
		key_chars_sz = strlen(KEY_CHARS);
		printf("+++UT:L%d: Test the 2000 numeric-string keys "
			"\"0000\" to \"1999\" "
			"(hash table depth = %ld)...\n"
			, __LINE__
			,			(long)depth
			); (void)fflush(stdout);
		errno	= 0; exit_code++;

		hashtable = create_hash_table(depth, NULL, cmp_keys
								, hint);
		if (hashtable == NULL) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"create_hash_table() "
				"returns %p(errno = %d).\n"
				,	__LINE__,	exit_code
				,	(void *)hashtable, errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

		/* insert and replace numeric key strings... */
		for (i[0] = 0; i[0] < 2; ++i[0]) /* '0' or '1' */
		for (i[1] = 0; i[1] < key_chars_sz; ++i[1])
		for (i[2] = 0; i[2] < key_chars_sz; ++i[2])
		for (i[3] = 0; i[3] < key_chars_sz; ++i[3]) {
			key0[0] = KEY_CHARS[i[0]];
			key0[1] = KEY_CHARS[i[1]];
			key0[2] = KEY_CHARS[i[2]];
			key0[3] = KEY_CHARS[i[3]];
			key0[4]	= '\0'; keys[0]	= key0;

			printf("UT:L%d: make_hashval(%p [%s], %lu, "
								"%ld) "
				"returns %ld.\n"
				, __LINE__, (void *)keys, keys[0]
				, (unsigned long)n_keys
				, (long)hashtable->depth
				, (long)make_hashval(keys
						, n_keys
						, hashtable->depth
						)
				); (void)fflush(stdout);

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table(hashtable = "
								"%p, "
					"keys[0] = %p (%s), %lu, "
					"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(
						  all_inserted_hashtable
						, keys
						, n_keys
						, (all_elem =
						    make_elem(keys
							, n_keys
							))
					)) != all_inserted_hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table("
					"all_inserted_hashtable  = %p, "
					"keys[0] = %p (%s), %lu, "
					"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					, (void *)all_inserted_hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)all_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = replace_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"replace_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, elem "
								"= %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			destroy_elem(&repl_elem, n_keys);

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			printf("UT:L%d: hashtable = %p. "
				"get_inserted_cnt_hash_table() "
				"returns %lu.\n"
				"keys[0] = %p (%s). "
				"get_duplicated_cnt_hash_table() "
				"returns %lu.\n"
				, __LINE__,	(void *)hashtable
				, (unsigned long)
				  get_inserted_cnt_hash_table(hashtable)
				,	keys[0], keys[0]
				, (unsigned long)
					get_duplicated_cnt_hash_table(
							  hashtable
							, keys
							, n_keys
							)
				); (void)fflush(stdout);
		}

		printf("UT:L%d: get_used_cnt_hash_table(%p) "
			"returns %ld.\n"
			, __LINE__,		(void *)hashtable
			,	(long)get_used_cnt_hash_table(hashtable)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		/* delete all stored datums... */
		for (i[0] = 0; i[0] < 2; ++i[0]) /* '0' or '1' */
		for (i[1] = 0; i[1] < key_chars_sz; ++i[1])
		for (i[2] = 0; i[2] < key_chars_sz; ++i[2])
		for (i[3] = 0; i[3] < key_chars_sz; ++i[3]) {
			key0[0] = KEY_CHARS[i[0]];
			key0[1] = KEY_CHARS[i[1]];
			key0[2] = KEY_CHARS[i[2]];
			key0[3] = KEY_CHARS[i[3]];
			key0[4]	= '\0'; keys[0]	= key0;

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = delete_hash_table(hashtable
						, keys
						, n_keys
						, repl_elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"delete_hash_table(hashtable = "
								"%p, "
					"keys[0] = %p (%s), %lu, %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}
		}

		printf("UT:L%d: hashtable = %p. "
			"get_inserted_cnt_hash_table() returns %lu.\n"
			, __LINE__,	(void *)hashtable
			, (unsigned long)get_inserted_cnt_hash_table(
							hashtable
							)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		state	= destroy_hash_table(hashtable);
		if (state != 0) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"destroy_hash_table() "
				"returns %d(errno = %d).\n"
				,	__LINE__,	exit_code
				,	state,	errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

#		undef	KEY_CHARS
		printf("+++UT:L%d: Test the 2000 numeric-string keys "
			"\"0000\" to \"1999\" done!\n\n"
			, __LINE__
			);(void)fflush(stdout);
	} /* Test the 2000 numeric-string keys "0000" to "1999" */

	/* Test the 2000 symbolic-string keys '!"#<' to '/~_;' */
	if (ac == 1 || strcmp(av[1], "-S") == 0) {
#		define	KEY_CHARS "!\"#$%&'()=~|`{+*}<>?_-^\\@[;:],./"
		key_chars_sz = strlen(KEY_CHARS);
		printf("+++UT:L%d: Test 2000 symbolic-string keys "
			"'!\"#<' to '/~_;' "
			"(hash table depth = %ld)...\n"
			, __LINE__
			,			(long)depth
			); (void)fflush(stdout);
		errno	= 0; exit_code++;

		hashtable = create_hash_table(depth, NULL, cmp_keys
								, hint);
		if (hashtable == NULL) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"create_hash_table() "
				"returns %p(errno = %d).\n"
				,	__LINE__,	exit_code
				,	(void *)hashtable, errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

		/* insert and replace symbol key strings... */
		for (i[0] = 0; i[0] < 32; i[0] += 31) /* '!', '/' */
		for (i[1] = 1; i[1] < 11; ++i[1]) /* '"' - '~' */
		for (i[2] = 2; i[2] < 22; i[2] += 2) /* '#','%'...'_' */
		for (i[3] = 17; i[3] < 27; ++i[3]) /* '<' - ';' */ {
			key0[0] = KEY_CHARS[i[0]];
			key0[1] = KEY_CHARS[i[1]];
			key0[2] = KEY_CHARS[i[2]];
			key0[3] = KEY_CHARS[i[3]];
			key0[4] = '\0'; keys[0] = key0;

			printf("UT:L%d: make_hashval(%p [%s], %lu, "
								"%ld) "
				"returns %ld.\n"
				, __LINE__, (void *)keys, keys[0]
				, (unsigned long)n_keys
				, (long)hashtable->depth
				, (long)make_hashval(keys
						, n_keys
						, hashtable->depth
						)
				); (void)fflush(stdout);

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table(hashtable = "
								"%p, "
					"keys[0] = %p (%s), %lu, "
							"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(
						  all_inserted_hashtable
						, keys
						, n_keys
						, (all_elem =
						    make_elem(keys
							, n_keys
							))
					)) != all_inserted_hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table("
					"all_inserted_hashtable  = %p, "
					"keys[0] = %p (%s), %lu, "
					"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					, (void *)all_inserted_hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)all_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = replace_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"replace_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, "
							"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			destroy_elem(&repl_elem, n_keys);

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			printf("UT:L%d: hashtable = %p. "
				"get_inserted_cnt_hash_table() "
				"returns %lu.\n"
				"keys[0] = %p (%s). "
				"get_duplicated_cnt_hash_table() "
				"returns %lu.\n"
				, __LINE__,	(void *)hashtable
				, (unsigned long)
				  get_inserted_cnt_hash_table(hashtable)
				,	keys[0], keys[0]
				, (unsigned long)
					get_duplicated_cnt_hash_table(
							hashtable
							, keys
							, n_keys
							)
				); (void)fflush(stdout);
		}

		printf("UT:L%d: get_used_cnt_hash_table(%p) "
			"returns %ld.\n"
			, __LINE__,		(void *)hashtable
			, (long)get_used_cnt_hash_table(hashtable)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		/* delete all stored datum... */
		for (i[0] = 0; i[0] < 32; i[0] += 31) /* '!', '/' */
		for (i[1] = 1; i[1] < 11; ++i[1]) /* '"' - '~' */
		for (i[2] = 2; i[2] < 22; i[2] += 2) /* '#','%'...'_' */
		for (i[3] = 17; i[3] < 27; ++i[3]) /* '<' - ';' */ {
			key0[0] = KEY_CHARS[i[0]];
			key0[1] = KEY_CHARS[i[1]];
			key0[2] = KEY_CHARS[i[2]];
			key0[3] = KEY_CHARS[i[3]];
			key0[4] = '\0'; keys[0] = key0;

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = delete_hash_table(hashtable
						, keys
						, n_keys
						, repl_elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"delete_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}
		}

		printf("UT:L%d: hashtable = %p. "
			"get_inserted_cnt_hash_table() returns %lu.\n"
			, __LINE__,	(void *)hashtable
			, (unsigned long)get_inserted_cnt_hash_table(
							hashtable
							)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		state	= destroy_hash_table(hashtable);
		if (state != 0) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"destroy_hash_table() "
				"returns %d(errno = %d).\n"
				,	__LINE__,	exit_code
				,	state,	errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

#		undef	KEY_CHARS
		printf("+++UT:L%d: Test 2000 symbolic-string keys "
			"'!\"#<' to '/~_;' done!\n\n"
			, __LINE__
			); (void)fflush(stdout);
	} /* Test the 2000 symbolic-string keys '!"#<' to '/~_;' */

	/* Test the 2000 alphabetic-string keys "abcn" to "zkuw" */
	if (ac == 1 || strcmp(av[1], "-A") == 0) {
#		define	KEY_CHARS	"abcdefghijklmnopqrstuvwxyz"
		key_chars_sz = strlen(KEY_CHARS);
		printf("+++UT:L%d: Test 2000 alphabetic-string keys "
			"\"abcn\" to \"zkuw\" "
			"(hash table depth = %ld)...\n"
			, __LINE__
			,			(long)depth
			); (void)fflush(stdout);
		errno	= 0; exit_code++;

		hashtable = create_hash_table(depth, NULL, cmp_keys
								, hint);
		if (hashtable == NULL) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"create_hash_table() "
				"returns %p(errno = %d).\n"
				,	__LINE__,	exit_code
				,	(void *)hashtable,	errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

		/* insert and replace the alphabetic key strings... */
		for (i[0] = 0; i[0] < 26; i[0] += 25) /* 'a', 'z' */
		for (i[1] = 1; i[1] < 11; ++i[1]) /* 'b' - 'k' */
		for (i[2] = 2; i[2] < 22; i[2] += 2) /* 'c','e'...'u' */
		for (i[3] = 13; i[3] < 23; ++i[3]) /* 'n' - 'w' */ {
			key0[0] = KEY_CHARS[i[0]];
			key0[1] = KEY_CHARS[i[1]];
			key0[2] = KEY_CHARS[i[2]];
			key0[3] = KEY_CHARS[i[3]];
			key0[4] = '\0'; keys[0] = key0;

			printf("UT:L%d: make_hashval(%p [%s], %lu, "
								"%ld) "
				"returns %ld.\n"
				, __LINE__, (void *)keys, keys[0]
				, (unsigned long)n_keys
				, (long)hashtable->depth
				, (long)make_hashval(keys
						, n_keys
						, hashtable->depth
						)
				); (void)fflush(stdout);

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, elem "
								"= %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(
						  all_inserted_hashtable
						, keys
						, n_keys
						, (all_elem =
						    make_elem(keys
							, n_keys
							))
					)) != all_inserted_hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table("
					"all_inserted_hashtable  = %p, "
					"keys[0] = %p (%s), %lu, "
					"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					, (void *)all_inserted_hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)all_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					,	(void *)repl_elem
					,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
						, (unsigned long)n_keys
					,	(void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = replace_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"replace_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, "
							"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			destroy_elem(&repl_elem, n_keys);

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					,	(unsigned long)n_keys
					,	(void *)repl_elem
					,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			printf("UT:L%d: hashtable = %p. "
				"get_inserted_cnt_hash_table() "
							"returns %lu.\n"
				"keys[0] = %p (%s). "
				"get_duplicated_cnt_hash_table() "
							"returns %lu.\n"
				, __LINE__,	(void *)hashtable
				, (unsigned long)
				  get_inserted_cnt_hash_table(hashtable)
				,	keys[0], keys[0]
				, (unsigned long)
					get_duplicated_cnt_hash_table(
							  hashtable
							, keys
							, n_keys
							)
				); (void)fflush(stdout);
		}

		printf("UT:L%d: get_used_cnt_hash_table(%p) "
			"returns %ld.\n"
			, __LINE__,		(void *)hashtable
			,	(long)get_used_cnt_hash_table(hashtable)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		/* delete the stored datum... */
		for (i[0] = 0; i[0] < 26; i[0] += 25) /* 'a', 'z' */
		for (i[1] = 1; i[1] < 11; ++i[1]) /* 'b' - 'k' */
		for (i[2] = 2; i[2] < 22; i[2] += 2) /* 'c','e'...'u' */
		for (i[3] = 13; i[3] < 23; ++i[3]) /* 'n' - 'w' */ {
			key0[0] = KEY_CHARS[i[0]];
			key0[1] = KEY_CHARS[i[1]];
			key0[2] = KEY_CHARS[i[2]];
			key0[3] = KEY_CHARS[i[3]];
			key0[4] = '\0'; keys[0] = key0;

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = delete_hash_table(hashtable
						, keys
						, n_keys
						, repl_elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"delete_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}
		}

		printf("UT:L%d: hashtable = %p. "
			"get_inserted_cnt_hash_table() returns %lu.\n"
			, __LINE__,	(void *)hashtable
			, (unsigned long)get_inserted_cnt_hash_table(
							hashtable
							)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		state	= destroy_hash_table(hashtable);
		if (state != 0) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"destroy_hash_table() "
				"returns %d(errno = %d).\n"
				,	__LINE__,	exit_code
				,	state,	errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

#		undef	KEY_CHARS
		printf("+++UT:L%d: Test 2000 alphabetic-string keys "
			"\"abcn\" to \"zkuw\" done!\n\n"
			, __LINE__
			); (void)fflush(stdout);
	} /* Test the 2000 alphabetic-string keys "aaa" to "czz" */

	/* Test the 2000 hankaku-kana-string keys "" to "ڎ" */
	if (ac == 1 || strcmp(av[1], "-H") == 0) {
#		define	KEY_CHARS				   \
			"" \
			"ÎĎŎƎǎȎɎʎˎ͎̎ΎώЎюҎ" \
			"ԎՎ֎׎؎َڎێ܎ݎގߎ"
		key_chars_sz = strlen(KEY_CHARS) / 2;
		printf("+++UT:L%d: Test 2000 hankaku-kana-string keys "
			"\"\" to \"ڎ\" "
			"(hash table depth = %ld)...\n"
			, __LINE__
			,			(long)depth
			); (void)fflush(stdout);
		errno	= 0; exit_code++;

		hashtable = create_hash_table(depth, NULL, cmp_keys
								, hint);
		if (hashtable == NULL) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"create_hash_table() "
				"returns %p(errno = %d).\n"
				,	__LINE__,	exit_code
				,	(void *)hashtable, errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

		/* insert and replace the hankaku-kana key strings... */
		for (i[0] = 0; i[0] < 50; ++i[0]) /* '' - '' */
		for (i[1] = 22; i[1] < 62; ++i[1]) /* '' - '' */ {
			memcpy(key0 + 0, KEY_CHARS + (i[0] * 2)
					, sizeof(short));
			memcpy(key0 + 2, KEY_CHARS + (i[1] * 2)
					, sizeof(short));
			key0[4] = '\0'; keys[0] = key0;

			printf("UT:L%d: make_hashval(%p [%s], %lu, "
								"%ld) "
				"returns %ld.\n"
				, __LINE__, (void *)keys, keys[0]
				, (unsigned long)n_keys
				, (long)hashtable->depth
				, (long)make_hashval(keys
						, n_keys
						, hashtable->depth
						)
				); (void)fflush(stdout);

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, "
							"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = insert_hash_table(
						  all_inserted_hashtable
						, keys
						, n_keys
						, (all_elem =
						    make_elem(keys
							, n_keys
							))
					)) != all_inserted_hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"insert_hash_table("
					"all_inserted_hashtable  = %p, "
					"keys[0] = %p (%s), %lu, "
					"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					, (void *)all_inserted_hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)all_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((elem = make_elem(keys, n_keys)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"make_elem(keys[0] = %p (%s), "
								"%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = replace_hash_table(hashtable
						, keys
						, n_keys
						, elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"replace_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, "
							"elem = %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			destroy_elem(&repl_elem, n_keys);

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) != elem) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			printf("UT:L%d: hashtable = %p. "
				"get_inserted_cnt_hash_table() "
				"returns %lu.\n"
				"keys[0] = %p (%s). "
				"get_duplicated_cnt_hash_table() "
				"returns %lu.\n"
				, __LINE__,	(void *)hashtable
				, (unsigned long)
				  get_inserted_cnt_hash_table(hashtable)
				,	keys[0], keys[0]
				, (unsigned long)
					get_duplicated_cnt_hash_table(
							  hashtable
							, keys
							, n_keys
							)
				); (void)fflush(stdout);
		}

		printf("UT:L%d: get_used_cnt_hash_table(%p) "
			"returns %ld.\n"
			, __LINE__,		(void *)hashtable
			,	(long)get_used_cnt_hash_table(hashtable)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		/* delete the stored datum... */
		for (i[0] = 0; i[0] < 50; ++i[0]) /* '' - '' */
		for (i[1] = 22; i[1] < 62; ++i[1]) /* '' - '' */ {
			memcpy(key0 + 0, KEY_CHARS + (i[0] * 2)
						, sizeof(short));
			memcpy(key0 + 2, KEY_CHARS + (i[1] * 2)
						, sizeof(short));
			key0[4] = '\0'; keys[0] = key0;

			if ((repl_elem = (char **)get_elem_hash_table(
							  hashtable
							, keys
							, n_keys
							)) == NULL) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"get_elem_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					,	keys[0], keys[0]
						, (unsigned long)n_keys
					,	(void *)repl_elem
					,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}

			if ((r_hashtable = delete_hash_table(hashtable
						, keys
						, n_keys
						, repl_elem
						)) != hashtable) {
				printf("!!FATAL!!:L%d:exit_code [%d]:"
					"delete_hash_table(hashtable "
								"= %p, "
					"keys[0] = %p (%s), %lu, %p) "
					"returns %p (errno = %d).\n"
					,	__LINE__, exit_code
					,	(void *)hashtable
					, keys[0], keys[0]
					, (unsigned long)n_keys
					, (void *)repl_elem
					, (void *)r_hashtable,	errno
					); (void)fflush(stdout);
				exit(exit_code);
			}
		}

		printf("UT:L%d: hashtable = %p. "
			"get_inserted_cnt_hash_table() returns %lu.\n"
			, __LINE__,	(void *)hashtable
			, (unsigned long)get_inserted_cnt_hash_table(
							hashtable
							)
			); (void)fflush(stdout);
		printf("UT:L%d: get_allocated_size_hash_table(%p) "
			"returns %lu "
			"(hashtable->used_cnt = %ld).\n"
			, __LINE__,		(void *)hashtable
			, (unsigned long)get_allocated_size_hash_table(
							hashtable)
			,		(long)hashtable->used_cnt
			); (void)fflush(stdout);

		state	= destroy_hash_table(hashtable);
		if (state != 0) {
			printf("!!FATAL!!:L%d:exit_code [%d]:"
				"destroy_hash_table() "
				"returns %d(errno = %d).\n"
				,	__LINE__,	exit_code
				,	state,	errno
				); (void)fflush(stderr);
			exit(exit_code);
		}

#		undef	KEY_CHARS
		printf("+++UT:L%d: Test 2000 hankaku-kana-string keys "
			"\"\" to \"ڎ\" done!\n\n"
			, __LINE__
			); (void)fflush(stdout);
	} /* Test the 2000 hankaku-kana-string keys "" to "ڎ" */

	/* test traversing 'all_inserted_hashtable'
	 * using 'get_elem_by_hashtable_cursor()'
	 */
	{	size_t	i;

		printf("++UT:L%d: traversing 'all_inserted_hashtable "
			"(addr = %p, depth = %ld, used_cnt = %ld, "
			"inserted_cnt = %lu, "
			"allocated size = %lu)'...\n"
			, __LINE__
			, (void *)all_inserted_hashtable
			, (long)(all_inserted_hashtable->depth)
			, (long)(all_inserted_hashtable->used_cnt)
			, (unsigned long)(all_inserted_hashtable
						->inserted_cnt)
			, (unsigned long)get_allocated_size_hash_table(
						all_inserted_hashtable)
			); (void)fflush(stdout);
		errno	= 0; exit_code++;

		for (i = 1, cursor = NULL
		; (repl_elem = (char **)get_elem_by_hashtable_cursor(
					  all_inserted_hashtable
					, &cursor
					, HASHTABLE_CURSOR_DIR_NEXT
					)) != NULL
		; ++i) {
			printf("UT:L%d:main(): %lu-th datum in "
				"'all_inserted_hashtable  %p'is "
				"%p (%s). "
				"cursor %p (HASHTABLE_CURSOR_DIR_NEXT) "
					"{table = %p, "
					"tables_idx = %ld, "
					"walked_tables_cnt = %ld, "
					"elements_idx = %ld}\n"
				, __LINE__,	(unsigned long)i
				,	(void *)all_inserted_hashtable
				,	(void *)repl_elem, repl_elem[0]
				,	(void *)cursor
				,	(void *)(cursor->table)
				,	(long)cursor->tables_idx
				,	(long)cursor->walked_tables_cnt
				,	(long)cursor->elements_idx
				); (void)fflush(stdout);
		}
		printf("\n"); (void)fflush(stdout);

		if ((repl_elem = (char **)get_elem_by_hashtable_cursor(
					  all_inserted_hashtable
					, &cursor
					, HASHTABLE_CURSOR_DIR_CURRENT
					)) != NULL) {
			printf("UT:L%d:main(): %lu-th datum in "
				"'all_inserted_hashtable  %p'is "
				"%p (%s). "
				"cursor %p "
				"(HASHTABLE_CURSOR_DIR_CURRENT) "
					"{table = %p, "
					"tables_idx = %ld, "
					"walked_tables_cnt = %ld, "
					"elements_idx = %ld}\n"
				, __LINE__,	(unsigned long)i
				,	(void *)all_inserted_hashtable
				,	(void *)repl_elem, repl_elem[0]
				,	(void *)cursor
				,	(void *)(cursor->table)
				,	(long)cursor->tables_idx
				,	(long)cursor->walked_tables_cnt
				,	(long)cursor->elements_idx
				); (void)fflush(stdout);
		}
		printf("\n"); (void)fflush(stdout);

		for (i = 1
		; (repl_elem = (char **)get_elem_by_hashtable_cursor(
					  all_inserted_hashtable
					, &cursor
					, HASHTABLE_CURSOR_DIR_PREVIOUS
					)) != NULL
		; ++i) {
			printf("UT:L%d:main(): %lu-th datum in "
				"'all_inserted_hashtable  %p'is "
				"%p (%s). "
				"cursor %p "
				"(HASHTABLE_CURSOR_DIR_PREVIOUS) "
				"{table = %p, "
					"tables_idx = %ld, "
					"walked_tables_cnt = %ld, "
					"elements_idx = %ld}\n"
				, __LINE__,	(unsigned long)i
				,	(void *)all_inserted_hashtable
				,	(void *)repl_elem, repl_elem[0]
				,	(void *)cursor
				,	(void *)(cursor->table)
				,	(long)cursor->tables_idx
				,	(long)cursor->walked_tables_cnt
				,	(long)cursor->elements_idx
				); (void)fflush(stdout);
		}
		printf("\n"); (void)fflush(stdout);

		if ((repl_elem = (char **)get_elem_by_hashtable_cursor(
					  all_inserted_hashtable
					, &cursor
					, HASHTABLE_CURSOR_DIR_CURRENT
					)) != NULL) {
			printf("UT:L%d:main(): %lu-th datum in "
				"'all_inserted_hashtable  %p'is "
				"%p (%s). "
				"cursor %p "
				"(HASHTABLE_CURSOR_DIR_CURRENT) "
					"{table = %p, "
					"tables_idx = %ld, "
					"walked_tables_cnt = %ld, "
					"elements_idx = %ld}\n"
				, __LINE__,	(unsigned long)i
				,	(void *)all_inserted_hashtable
				,	(void *)repl_elem, repl_elem[0]
				,	(void *)cursor
				,	(void *)(cursor->table)
				,	(long)cursor->tables_idx
				,	(long)cursor->walked_tables_cnt
				,	(long)cursor->elements_idx
				); (void)fflush(stdout);
		}

		(void)get_elem_by_hashtable_cursor(
					  all_inserted_hashtable
					, &cursor
					, HASHTABLE_CURSOR_DIR_DESTROY
					);

		/* destroy 'all_inserted_hashtable' */
		state	= destroy_hash_table(all_inserted_hashtable);
		err_num	= errno;
		printf("UT:L%d: destroy_hash_table(%p) "
			"returns %d. 'errno' = %d.\n"
			, __LINE__,	(void *)all_inserted_hashtable
			,	state,		err_num
		); (void)fflush(stdout);
		if (state != 0 || err_num != 0) {
			exit(exit_code);
		}
		errno	= 0;

		printf("++UT:L%d: traversing 'all_inserted_hashtable "
			"done!\n\n"
			, __LINE__
			); (void)fflush(stdout);
	} /* test traversing 'all_inserted_hashtable'
	   * using 'get_elem_by_hashtable_cursor()'
	   */

	/* Test performance */
	if (ac == 1 || strcmp(av[1], "-P") == 0) {
#		define	KEY_CHARS				\
				  "aycxewgtirkpnmubzdvfshqjol"	\
				  "9081726354"			\
				  "YAXCWETGRIPKMNBUDZFVHSJQLO"  \
				  "_!?\">#<$}%*&+'{(`)|~=/.-,^]:@;["

		key_chars_sz = strlen(KEY_CHARS);
		printf("+++UT:L%d: Performance test "
			"(hash table depth = %ld)...\n"
			, __LINE__
			,			(long)depth
			); (void)fflush(stdout);
		errno	= 0; exit_code++;

		for ( j = 0, hint = NULL
		; j <= (sizeof(hints) / sizeof(hints[0]))
		; hint = hints + j++) {
		for (limit = 1, set_hint_max_insert_cnt = 0
		; limit <= 10
		; ++limit) {
			if ((keypool = (char *)malloc(KEY_LEN
						* GET_KEYS_N(limit
							, key_chars_sz))
						) == NULL) {
				printf("!!FATAL!!:L%d:perf#%lu:"
					"exit_code [%d]: "
					"malloc(%lu) "
					"returns %p (errno = %d).\n"
					,	__LINE__
					,		(unsigned long)j
					,	exit_code
					, (unsigned long)(KEY_LEN
						* GET_KEYS_N(limit
							, key_chars_sz))
					,	keypool,	errno
					); (void)fflush(stderr);
				exit(exit_code);
			}
			keyp	= keypool;
				
			if (((hint != NULL)
				&& hint->max_insert_cnt
					== (ssize_t)(LONG_MIN + 1L))
			|| set_hint_max_insert_cnt != 0) {
				set_hint_max_insert_cnt	= 1;
				hint->max_insert_cnt = (ssize_t)
					GET_KEYS_N(limit, key_chars_sz);
			}
			if (limit == 1) {
				printf("UT:L%d: performance test #%lu "
				"(hint (%p) {"
				"max_insert_cnt = %ld. "
				"elems_base_cnt = %ld. "
				"elems_ext_cnt = %ld.} "
				"starts...\n"
				, __LINE__, 	(unsigned long)j
				,	(void *)hint
				, (long)((hint != NULL)?
						  hint->max_insert_cnt
						: -999)
				, (long)((hint != NULL)?
						  hint->elems_base_cnt
						: -999)
				, (long)((hint != NULL)?
						  hint->elems_ext_cnt
						: -999)
				); (void)fflush(stdout);
			}

			hashtable = create_hash_table(depth, NULL
						, cmp_keys, hint);
			if (hashtable == NULL) {
				printf("!!FATAL!!:L%d:perf#%lu:"
					"exit_code [%d]: "
					"create_hash_table() "
					"returns %p(errno = %d).\n"
					,	__LINE__
					,		(unsigned long)j
					,	exit_code
					,	(void *)hashtable
					,	errno
					); (void)fflush(stderr);
				exit(exit_code);
			}

			/* insert and replace many key strings... */
			gettimeofday(&start_tv, NULL);
						/* get start time */
			for (i[0] = 0; i[0] < 2; ++i[0])
			for (i[1] = 0; i[1] < 4 * limit; i[1] += 4)
			for (i[2] = 0; i[2] < key_chars_sz; ++i[2])
			for (i[3] = 0; i[3] < key_chars_sz; ++i[3]) {
				*(keyp)		= KEY_CHARS[i[0]];
				*(keyp + 1)	= KEY_CHARS[i[1]];
				*(keyp + 2)	= KEY_CHARS[i[2]];
				*(keyp + 3)	= KEY_CHARS[i[3]];
				*(keyp + 4)	= '\0'; keys[0] = keyp;

				if ((elem = make_elem(&keyp, n_keys))
							== NULL) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]:"
						"make_elem(&keyp = "
						"%p (%s), %lu) "
						"returns %p "
						"(errno = %d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						,	(void *)elem
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

				if ((r_hashtable
					= insert_hash_table(hashtable
						, &keyp
						, n_keys
						, elem
						)) != hashtable) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]:"
						"insert_hash_table("
						"hashtable = %p, "
						"&keyp = %p (%s), "
						"%lu, elem = %p) "
						"returns %p (errno = "
						"%d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)hashtable
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)elem
						, (void *)r_hashtable
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

				keyp	+= KEY_LEN;
			}
			gettimeofday(&end_tv, NULL); /* get end time */

			elapsed_tv.tv_sec
				= end_tv.tv_sec - start_tv.tv_sec;
			elapsed_tv.tv_usec
				= end_tv.tv_usec - start_tv.tv_usec;
			if (elapsed_tv.tv_usec < 0L) {
				--elapsed_tv.tv_sec;
				elapsed_tv.tv_usec += 1000000L;
			}
			printf("UT:L%d:perf#%lu:insert_hash_table():"
				"%lu datum: "
				"elapsed time = %ld.%06ld. "
				"(start time = %ld.%06ld, "
				"end time = %ld.%06ld)\n"
				, __LINE__, (unsigned long)j
				, (unsigned long)GET_KEYS_N(limit
							, key_chars_sz
							)
				, elapsed_tv.tv_sec, elapsed_tv.tv_usec
				, start_tv.tv_sec, start_tv.tv_usec
				, end_tv.tv_sec, end_tv.tv_usec
				); (void)fflush(stdout);

			printf("UT:L%d:perf#%lu: "
				"get_used_cnt_hash_table(%p) returns "
				"%ld.\n"
				, __LINE__, (unsigned long)j
				, (void *)hashtable
				, (long)get_used_cnt_hash_table(
							hashtable
							)
				); (void)fflush(stdout);

			printf("UT:L%d:perf#%lu: after all "
				"insert_hash_table() hashtable = %p. "
				"get_inserted_cnt_hash_table() "
				"returns %lu.\n"
				, __LINE__, (unsigned long)j
				,		(void *)hashtable
				, (unsigned long)
					get_inserted_cnt_hash_table(
							hashtable
							)
				); (void)fflush(stdout);
			printf("UT:L%d:perf#%lu: "
				"get_allocated_size_hash_table(%p) "
				"returns %lu "
				"(hashtable {used_cnt = %ld, "
				"max_insert_cnt = %ld, "
				"alloc_base_cnt = %ld, "
				"alloc_ext_cnt = %ld}).\n"
				, __LINE__, (unsigned long)j
				,		(void *)hashtable
				, (unsigned long)(
					get_allocated_size_hash_table(
							hashtable))
				,	(long)hashtable->used_cnt
				,	(long)hashtable->max_insert_cnt
				,	(long)hashtable->alloc_base_cnt
				,	(long)hashtable->alloc_ext_cnt
				); (void)fflush(stdout);


			/* replace many key strings... */
			gettimeofday(&start_tv, NULL);
						/* get start time */
			for (i[0] = 0, keyp = keypool
			; i[0] < GET_KEYS_N(limit, key_chars_sz)
			; keyp += KEY_LEN, ++i[0]) {
				if ((repl_elem = (char **)
					get_elem_hash_table(hashtable
							, &keyp
							, n_keys
							)) == NULL) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]: "
						"get_elem_hash_table("
						"hashtable = %p, "
						"&keyp = %p (%s), "
						"%lu) returns %p "
						"(errno = %d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)hashtable
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)repl_elem
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

				if ((elem = make_elem(&keyp, n_keys))
							== NULL) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]: "
						"make_elem(&keyp = "
						"%p (%s), %lu) "
						"returns %p (errno = "
						"%d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)elem,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

				if ((r_hashtable
					= replace_hash_table(hashtable
						, &keyp
						, n_keys
						, elem
						)) != hashtable) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]: "
						"replace_hash_table("
						"hashtable = %p, "
						"&keyp = %p (%s), "
						"%lu, elem = %p) "
						"returns %p "
						"(errno = %d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)hashtable
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)elem
						, (void *)r_hashtable
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

				destroy_elem(&repl_elem, n_keys);

				if ((repl_elem = (char **)
						get_elem_hash_table(
							hashtable
							, &keyp
							, n_keys
							)) != elem) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]: "
						"get_elem_hash_table("
						"hashtable = %p, "
						"&keyp = %p (%s), "
						"%lu) returns %p "
						"(errno = %d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)hashtable
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)repl_elem
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

			}
			elem	= NULL; /* clear the pointer */
			gettimeofday(&end_tv, NULL); /* get end time */

			elapsed_tv.tv_sec
				= end_tv.tv_sec - start_tv.tv_sec;
			elapsed_tv.tv_usec
				= end_tv.tv_usec - start_tv.tv_usec;
			if (elapsed_tv.tv_usec < 0L) {
				--elapsed_tv.tv_sec;
				elapsed_tv.tv_usec += 1000000L;
			}
			printf("UT:L%d:perf#%lu:replace_hash_table():"
				"%lu datum: "
				"elapsed time = %ld.%06ld. "
				"(start time = %ld.%06ld, "
				"end time = %ld.%06ld)\n"
				, __LINE__, (unsigned long)j
				, (unsigned long)GET_KEYS_N(limit
							, key_chars_sz
							)
				, elapsed_tv.tv_sec, elapsed_tv.tv_usec
				, start_tv.tv_sec, start_tv.tv_usec
				, end_tv.tv_sec, end_tv.tv_usec
				); (void)fflush(stdout);

			printf("UT:L%d:perf#%lu: "
				"get_used_cnt_hash_table(%p) returns "
				"%ld.\n"
				, __LINE__, (unsigned long)j
				, (void *)hashtable
				, (long)get_used_cnt_hash_table(
							hashtable
							)
				); (void)fflush(stdout);

			printf("UT:L%d:perf#%lu: after all "
				"replace_hash_table() hashtable = %p. "
				"get_inserted_cnt_hash_table() "
				"returns %lu.\n"
				, __LINE__, (unsigned long)j
				,		(void *)hashtable
				, (unsigned long)
					get_inserted_cnt_hash_table(
							hashtable
							)
				); (void)fflush(stdout);
			printf("UT:L%d:perf#%lu: "
				"get_allocated_size_hash_table(%p) "
				"returns %lu "
				"(hashtable {used_cnt = %ld, "
				"max_insert_cnt = %ld, "
				"alloc_base_cnt = %ld, "
				"alloc_ext_cnt = %ld}).\n"
				, __LINE__, (unsigned long)j
				, (void *)hashtable
				, (unsigned long)(
					get_allocated_size_hash_table(
							hashtable))
				,	(long)hashtable->used_cnt
				,	(long)hashtable->max_insert_cnt
				,	(long)hashtable->alloc_base_cnt
				,	(long)hashtable->alloc_ext_cnt
				); (void)fflush(stdout);

			/* get the stored datum... */
			gettimeofday(&start_tv, NULL);
						/* get start time */
			for (i[0] = 0, keyp = keypool
			; i[0] < GET_KEYS_N(limit, key_chars_sz)
			; keyp += KEY_LEN, ++i[0]) {
				if ((repl_elem
					= (char **)get_elem_hash_table(
							  hashtable
							, &keyp
							, n_keys
							)) == NULL) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]: "
						"get_elem_hash_table("
						"hashtable = %p, "
						"&keyp = %p (%s), "
						"%lu) returns %p "
						"(errno = %d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)hashtable
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)repl_elem
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}
			}
			gettimeofday(&end_tv, NULL); /* get end time */

			elapsed_tv.tv_sec
				= end_tv.tv_sec - start_tv.tv_sec;
			elapsed_tv.tv_usec	
				= end_tv.tv_usec - start_tv.tv_usec;
			if (elapsed_tv.tv_usec < 0L) {
				--elapsed_tv.tv_sec;
				elapsed_tv.tv_usec += 1000000L;
			}
			printf("UT:L%d:perf#%lu:get_elem_table():"
				"%lu datum: "
				"elapsed time = %ld.%06ld. "
				"(start time = %ld.%06ld, "
				"end time = %ld.%06ld)\n"
				, __LINE__, (unsigned long)j
				, (unsigned long)GET_KEYS_N(limit
							, key_chars_sz
							)
				, elapsed_tv.tv_sec, elapsed_tv.tv_usec
				, start_tv.tv_sec, start_tv.tv_usec
				, end_tv.tv_sec, end_tv.tv_usec
				); (void)fflush(stdout);

			printf("UT:L%d:perf#%lu:after all "
				"get_elem_hash_table():hashtable = %p: "
				"get_inserted_cnt_hash_table() "
				"returns %lu.\n"
				, __LINE__, (unsigned long)j
				,		(void *)hashtable
				, (unsigned long)
					get_inserted_cnt_hash_table(
							hashtable
							)
				); (void)fflush(stdout);
			printf("UT:L%d:perf#%lu: "
				"get_allocated_size_hash_table(%p"
				") returns %lu "
				"(hashtable {used_cnt = %ld, "
				"max_insert_cnt = %ld, "
				"alloc_base_cnt = %ld, "
				"alloc_ext_cnt = %ld}).\n"
				, __LINE__, (unsigned long)j
				, (void *)hashtable
				, (unsigned long)(
					get_allocated_size_hash_table(
							hashtable))
				,	(long)hashtable->used_cnt
				,	(long)hashtable->max_insert_cnt
				,	(long)hashtable->alloc_base_cnt
				,	(long)hashtable->alloc_ext_cnt
				); (void)fflush(stdout);

			/* delete the stored datum... */
			gettimeofday(&start_tv, NULL);
						/* get start time */
			for (i[0] = 0, keyp = keypool
			; i[0] < GET_KEYS_N(limit, key_chars_sz)
			; keyp += KEY_LEN, ++i[0]) {
				if ((repl_elem
					= (char **)get_elem_hash_table(
							  hashtable
							, &keyp
							, n_keys
							)) == NULL) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]: "
						"get_elem_hash_table("
						"hashtable = %p, "
						"&keyp = %p (%s), "
						"%lu) returns %p "
						"(errno = %d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)hashtable
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)repl_elem
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

				if ((r_hashtable
					= delete_hash_table(hashtable
						, &keyp
						, n_keys
						, repl_elem
						)) != hashtable) {
					printf("!!FATAL!!:L%d:perf#%lu:"
						"exit_code [%d]: "
						"delete_hash_table("
						"hashtable = %p, "
						"&keyp = %p (%s), "
						"%lu, %p) returns %p "
						"(errno = %d).\n"
						, __LINE__
						, (unsigned long)j
						,	exit_code
						, (void *)hashtable
						, (void *)&keyp, keyp
						, (unsigned long)n_keys
						, (void *)repl_elem
						, (void *)r_hashtable
						,	errno
						); (void)fflush(stdout);
					exit(exit_code);
				}

				destroy_elem(&repl_elem, n_keys);
			}
			gettimeofday(&end_tv, NULL); /* get end time */

			elapsed_tv.tv_sec
				= end_tv.tv_sec - start_tv.tv_sec;
			elapsed_tv.tv_usec	
				= end_tv.tv_usec - start_tv.tv_usec;
			if (elapsed_tv.tv_usec < 0L) {
				--elapsed_tv.tv_sec;
				elapsed_tv.tv_usec += 1000000L;
			}
			printf("UT:L%d:perf#%lu:delete_hash_table():"
				"%lu datum: "
				"elapsed time = %ld.%06ld. "
				"(start time = %ld.%06ld, "
				"end time = %ld.%06ld)\n"
				, __LINE__, (unsigned long)j
				, (unsigned long)GET_KEYS_N(limit
							, key_chars_sz
							)
				, elapsed_tv.tv_sec, elapsed_tv.tv_usec
				, start_tv.tv_sec, start_tv.tv_usec
				, end_tv.tv_sec, end_tv.tv_usec
				); (void)fflush(stdout);

			printf("UT:L%d:perf#%lu:after all "
				"delete_hash_table():hashtable = %p: "
				"get_inserted_cnt_hash_table() "
				"returns %lu.\n"
				, __LINE__, (unsigned long)j
				,		(void *)hashtable
				, (unsigned long)
					get_inserted_cnt_hash_table(
							hashtable
							)
				); (void)fflush(stdout);
			printf("UT:L%d:perf#%lu: "
				"get_allocated_size_hash_table(%p"
				") returns %lu "
				"(hashtable {used_cnt = %ld, "
				"max_insert_cnt = %ld, "
				"alloc_base_cnt = %ld, "
				"alloc_ext_cnt = %ld}).\n"
				, __LINE__, (unsigned long)j
				, (void *)hashtable
				, (unsigned long)(
					get_allocated_size_hash_table(
							hashtable))
				,	(long)hashtable->used_cnt
				,	(long)hashtable->max_insert_cnt
				,	(long)hashtable->alloc_base_cnt
				,	(long)hashtable->alloc_ext_cnt
				); (void)fflush(stdout);

			state	= destroy_hash_table(hashtable);
			if (state != 0) {
				printf("!!FATAL!!:L%d:perf#%lu:"
					"exit_code [%d]: "
					"destroy_hash_table() "
					"returns %d(errno = %d).\n"
					, __LINE__, (unsigned long)j
					,	exit_code
					,	state,	errno
					); (void)fflush(stderr);
				exit(exit_code);
			}

			if (limit == 10) {
				printf("UT:L%d: performance test #%lu "
				"(hint = %p) done!\n\n"
				,	__LINE__,	(unsigned long)j
				,	(void *)hint
				); (void)fflush(stdout);
			}
		} free(keypool); }

#		undef	KEY_CHARS
		printf("+++UT:L%d: Performance test 172980 datum "
			"operations\n\n"
			, __LINE__
			); (void)fflush(stdout);
	} /* Test performance of the 172980 datum operations */

	if (elem != NULL) destroy_elem(&elem, n_keys); /* for safety */
	printf("+++UT: All tests are succeeded!\n")
	; (void)fflush(stdout);

	exit(0); /* All unit tests are okay! */
}
